@@ -14,7 +14,7 @@ use File::Copy qw(copy);
 use File::Path qw(mkpath rmtree);
 use B qw(perlstring);
 
-our $VERSION = '0.009018'; # 0.009.017
+our $VERSION = '0.010000'; # 0.10.0
 
 $VERSION = eval $VERSION;
 
@@ -200,67 +200,119 @@ sub script_command_file {
 
 sub fatpack_file {
   my ($self, $file) = @_;
-  my $cwd = cwd;
-  my @dirs = grep -d, map rel2abs($_, $cwd), ('lib','fatlib');
-  my %files;
-  foreach my $dir (@dirs) {
-    find(sub {
-      return unless -f $_;
-      !/\.pm$/ and warn "File ${File::Find::name} isn't a .pm file - can't pack this -- if you hoped we were going to, things may not be what you expected later\n" and return;
-      $files{File::Spec::Unix->abs2rel($File::Find::name,$dir)} = do {
-        local (@ARGV, $/) = ($File::Find::name); <>
-      };
-      close ARGV;
-    }, $dir);
+
+  my $shebang = "";
+  my $script = "";
+  if ( defined $file and -r $file ) {
+    ($shebang, $script) = $self->load_main_script($file);
   }
-  my $start = stripspace <<'  END_START';
+
+  my @dirs = $self->collect_dirs();
+  my %files;
+  $self->collect_files($_, \%files) for @dirs;
+
+  return join "\n", $shebang, $self->fatpack_code(\%files), $script;
+}
+
+# This method can be overload in sub classes
+# For example to skip POD
+sub load_file {
+  my ($self, $file) = @_;
+  my $content = do {
+    local (@ARGV, $/) = ($file);
+    <>
+  };
+  close ARGV;
+  return $content;
+}
+
+sub collect_dirs {
+  my ($self) = @_;
+  my $cwd = cwd;
+  return grep -d, map rel2abs($_, $cwd), ('lib','fatlib');
+}
+
+sub collect_files {
+  my ($self, $dir, $files) = @_;
+  find(sub {
+    return unless -f $_;
+    !/\.pm$/ and warn "File ${File::Find::name} isn't a .pm file - can't pack this -- if you hoped we were going to, things may not be what you expected later\n" and return;
+    $files->{File::Spec::Unix->abs2rel($File::Find::name,$dir)} =
+      $self->load_file($File::Find::name);
+  }, $dir);
+}
+
+sub load_main_script {
+  my ($self, $file) = @_;
+  open my $fh, "<", $file or die("Can't read $file: $!");
+  my $shebang = <$fh>;
+  my $script = join "", <$fh>;
+  close $fh;
+  unless ( index($shebang, '#!') == 0 ) {
+    $script = $shebang . $script;
+    $shebang = "";
+  }
+  return ($shebang, $script);
+}
+
+sub fatpack_start {
+  return stripspace <<'  END_START';
     # This chunk of stuff was generated by App::FatPacker. To find the original
     # file's code, look for the end of this BEGIN block or the string 'FATPACK'
     BEGIN {
     my %fatpacked;
   END_START
-  my $end = stripspace <<'  END_END';
+}
+
+sub fatpack_end {
+  return stripspace <<'  END_END';
     s/^  //mg for values %fatpacked;
 
-    unshift @INC, sub {
-      if (my $fat = $fatpacked{$_[1]}) {
-        if ($] < 5.008) {
-          return sub {
-            return 0 unless length $fat;
-            $fat =~ s/^([^\n]*\n?)//;
-            $_ = $1;
-            return 1;
-          };
-        }
-        open my $fh, '<', \$fat
-          or die "FatPacker error loading $_[1] (could be a perl installation issue?)";
-        return $fh;
-      }
-      return
-    };
+    my $class = 'FatPacked::'.(0+\%fatpacked);
+    no strict 'refs';
+    *{"${class}::files"} = sub { keys %{$_[0]} };
 
-    } # END OF FATPACK CODE
+    if ($] < 5.008) {
+      *{"${class}::INC"} = sub {
+         if (my $fat = $_[0]{$_[1]}) {
+           return sub {
+             return 0 unless length $fat;
+             $fat =~ s/^([^\n]*\n?)//;
+             $_ = $1;
+             return 1;
+           };
+         }
+         return;
+      };
+    }
+
+    else {
+      *{"${class}::INC"} = sub {
+        if (my $fat = $_[0]{$_[1]}) {
+          open my $fh, '<', \$fat
+            or die "FatPacker error loading $_[1] (could be a perl installation issue?)";
+          return $fh;
+        }
+        return;
+      };
+    }
+
+    unshift @INC, bless \%fatpacked, $class;
+  } # END OF FATPACK CODE
   END_END
+}
+
+sub fatpack_code {
+  my ($self, $files) = @_;
   my @segments = map {
     (my $stub = $_) =~ s/\.pm$//;
     my $name = uc join '_', split '/', $stub;
-    my $data = $files{$_}; $data =~ s/^/  /mg; $data =~ s/(?<!\n)\z/\n/;
-    '$fatpacked{'.perlstring($_).qq!} = <<'${name}';\n!
+    my $data = $files->{$_}; $data =~ s/^/  /mg; $data =~ s/(?<!\n)\z/\n/;
+    '$fatpacked{'.perlstring($_).qq!} = '#line '.(1+__LINE__).' "'.__FILE__."\\"\\n".<<'${name}';\n!
     .qq!${data}${name}\n!;
-  } sort keys %files;
-  my $shebang = "";
-  my $script = "";
-  if ( defined $file and -r $file ) {
-    open my $fh, "<", $file or die("Can't read $file: $!");
-    $shebang = <$fh>;
-    $script = join "", <$fh>;
-    close $fh;
-    unless ( index($shebang, '#!') == 0 ) {
-      $script = $shebang . $script;
-      $shebang = "";
-    }
-  }
-  return join "\n", $shebang, $start, @segments, $end, $script;
+  } sort keys %$files;
+
+  return join "\n", $self->fatpack_start, @segments, $self->fatpack_end;
 }
 
 =encoding UTF-8
@@ -282,7 +334,7 @@ Or, with more step-by-step control:
 
 See the documentation for the L<fatpack> script itself for more information.
 
-The programmatic API for this code is not yet fully decided, hence the 0.9
+The programmatic API for this code is not yet fully decided, hence the 0.x
 release version. Expect that to be cleaned up for 1.0.
 
 =head1 SEE ALSO
@@ -316,6 +368,12 @@ ether - Karen Etheridge (cpan:ETHER) <ether@cpan.org>
 
 Mithaldu - Christian Walde (cpan:MITHALDU) <walde.christian@googlemail.com>
 
+dolmen - Olivier Mengué (cpan:DOLMEN) <dolmen@cpan.org>
+
+djerius - Diab Jerius (cpan:DJERIUS) <djerius@cpan.org>
+
+haarg - Graham Knop (cpan:HAARG> <haarg@haarg.org>
+
 Many more people are probably owed thanks for ideas. Yet
 another doc nit to fix.
 
